<?php

namespace Drupal\simple_menu_tree\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a 'SimpleMenuTree' block.
 *
 * @Block(
 *  id = "simple_menu_tree",
 *  admin_label = @Translation("Simple Menu Tree"),
 *  category = @Translation("Component")
 * )
 */
class SimpleMenuTree extends BlockBase {

  public function getComponentName() {
    return 'simple-menu-tree:simple-menu-tree';
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'heading' => 'Menu',
      'source' => [
        'type' => 'menu',
        'value' => 'main',
        'depth' => 3,
      ],
      'layout' => 'vertical',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form['#parents'] = ['settings', 'simple_menu_tree'];

    $form['heading'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Heading'),
      '#description' => $this->t('The header title of the menu tree'),
      '#default_value' => $this->configuration['heading'],
    ];
    $form['source_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Source Type'),
      '#description' => $this->t('The source of the menu tree'),
      '#options' => [
        'menu' => $this->t('Menu'),
        'taxonomy' => $this->t('Taxonomy Term'),
      ],
      '#default_value' => $this->configuration['source']['type'],
      '#ajax' => [
        'callback' => [static::class, 'ajaxSourceTypeChange'],
        'wrapper' => 'source-value-wrapper',
      ],
    ];
    $options = [];

    $form['source_value'] = [
      '#prefix' => '<div id="source-value-wrapper">',
      '#suffix' => '</div>',
      '#type' => 'select',
      '#title' => $this->t('Source Value'),
      '#description' => $this->t('The value of the source, should be a menu name or taxonomy term id'),
      '#default_value' => $this->configuration['source']['value'],
      '#options' => $options,
      '#validated' => TRUE,
    ];

    $opts = range(1, 10);
    $form['source_depth'] = [
      '#type' => 'select',
      '#title' => $this->t('Depth'),
      '#description' => $this->t('The maximun depth of the menu tree to be displayed'),
      '#options' => array_combine($opts, $opts),
      '#default_value' => $this->configuration['source']['depth'],
    ];

    $form['layout'] = [
      '#type' => 'select',
      '#title' => $this->t('Layout'),
      '#options' => [
        'vertical' => $this->t('Vertical'),
        'horizontal' => $this->t('Horizontal'),
      ],
      '#default_value' => $this->configuration['layout'],
    ];

    $form['#process'] = [[static::class, 'processForm']];
    $form_state->set('configuration', $this->configuration);

    return $form;
  }

  /**
   * Generate the source value options based on the source type.
   */
  public static function processForm(array $element, FormStateInterface $form_state) {
    $configuration = $form_state->get('configuration');

    $source_type = $configuration['source']['type'];
    $user_input = @$form_state->getUserInput()['settings']['simple_menu_tree']['source_type'];
    if (!empty($user_input)) {
      $source_type = $user_input;
    }

    $options = $source_type == 'menu' ? static::getMenuOptions() : static::getTaxonomyOptions();
    $element['source_value']['#options'] = $options;
    unset($element['source_value']['#value']);
    return $element;
  }

  /**
   * Ajax callback for the source type change.
   */
  public static function ajaxSourceTypeChange(array &$form, FormStateInterface $form_state) {
    $source_type = $form_state->get(['settings', 'simple_menu_tree', 'source_type']);
    $form_state->setValue(['settings', 'simple_menu_tree', 'source_value'], NULL);
    $form_state->setRebuild();
    return $form['settings']['source_value'];
  }

  /**
   * Get the menu options.
   */
  protected static function getMenuOptions() {
    $options = [];
    $menus = \Drupal::entityTypeManager()->getStorage('menu')->loadMultiple();
    foreach ($menus as $menu) {
      $options[$menu->id()] = $menu->label();
    }
    return $options;
  }

  /**
   * Get the taxonomy options.
   */
  protected static function getTaxonomyOptions() {
    $options = [];
    $vocabularies = \Drupal::entityTypeManager()->getStorage('taxonomy_vocabulary')->loadMultiple();
    foreach ($vocabularies as $vocabulary) {
      $options[$vocabulary->id()] = $vocabulary->label();
    }
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [
      'component' => [
        '#type' => 'component',
        '#component' => 'simple_menu_tree:simple-menu-tree',
        '#props' => [
          ...$this->configuration,
        ],
      ],
      '#cache' => [
        'contexts' => ['url.path'],
        'tags' => ['config:simple_menu_tree.settings'],
      ],
    ];
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['title'] = $form_state->getValue('title');
    $this->configuration['source']['type'] = $form_state->getValue('source_type');
    $this->configuration['source']['value'] = $form_state->getValue('source_value');
    $this->configuration['source']['depth'] = $form_state->getValue('source_depth');
    $this->configuration['layout'] = $form_state->getValue('layout');
  }
}
