<?php

declare(strict_types=1);

namespace Drupal\simple_menu_tree;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Menu\MenuLinkTreeInterface;
use Drupal\Core\Menu\MenuTreeParameters;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;

/**
 * Twig extension.
 */
final class SimpleMenuTreeTwigExtension extends AbstractExtension {

  /**
   * Constructs the extension object.
   */
  public function __construct(
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly MenuLinkTreeInterface $menuLinkTree,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function getFunctions(): array {
    $functions[] = new TwigFunction('simple_menu_tree_load', [static::class, 'simpleMenuTreeLoad']);
    return $functions;
  }

  public static function simpleMenuTreeLoad($type, $id, $depth) {
    if ($type === 'menu') {
      $handle = \Drupal::menuTree();
      $params = new MenuTreeParameters();
      $params->setMinDepth(1)->setMaxDepth($depth);
      $menu = $handle->load($id, $params);

      $manipulators = [
        ['callable' => 'menu.default_tree_manipulators:checkNodeAccess'],
        ['callable' => 'menu.default_tree_manipulators:checkAccess'],
        ['callable' => 'menu.default_tree_manipulators:generateIndexAndSort']
      ];

      $tree = $handle->transform($menu, $manipulators);
      $menu = $handle->build($tree);

      if (!isset($menu['#items']) || !is_array($menu['#items'])) {
        return [];
      }

      $items = $menu['#items'];
      if (!empty($items)) {
        static::findLastActiveTrail($items);
      }

      return static::menuLinkTree($items);
    }

    if ($type === 'taxonomy') {
      $termStorage = \Drupal::entityTypeManager()->getStorage('taxonomy_term');
      $root = $termStorage->load(0);
      $vid = $vid ?? ($root ? $root->bundle() : 'category');

      $terms = $termStorage->loadTree($id, 0, NULL, TRUE);
      $data = [];

      foreach ($terms as $term) {
        $data[] = [
          'id' => $term->id(),
          'pid' => $term->parent->target_id,
          'title' => $term->label(),
          'url' => $term->toUrl()->toString(),
          'depth' => $term->depth
        ];
      }

      return static::buildTree($data);
    }

    return [];
  }

  protected static function findLastActiveTrail(&$items) {
    $return = true;
    foreach ($items as &$item) {
      if ($item['in_active_trail']) {
        if (!empty($item['below'])) {
          if (static::findLastActiveTrail($item['below'])) {
            $item['is_last_active_trail'] = true;
          }
        } else {
          $item['is_last_active_trail'] = true;
        }
        return false;
      }
    }
    return $return;
  }

  protected static function menuLinkTree($items, $depth = 1) {
    if (!is_array($items)) {
      return [];
    }

    $menu_links = [];
    foreach ($items as $item) {
      if (!is_array($item) || !isset($item['title']) || !isset($item['url'])) {
        continue;
      }

      $tmp = [
        'title' => $item['title'],
        'url' => $item['url']->toString(),
        'depth' => $depth,
        'options' => $item['url']->getOptions(),
        'in_active_trail' => $item['in_active_trail'] ?? false,
      ];

      // Add is_last_active_trail if it exists
      if (isset($item['is_last_active_trail'])) {
        $tmp['is_last_active_trail'] = $item['is_last_active_trail'];
      }

      if (!empty($item['below']) && is_array($item['below'])) {
        $tmp['children'] = static::menuLinkTree($item['below'], $depth + 1);
      }

      $menu_links[] = $tmp;
    }
    return $menu_links;
  }

  /**
   * @param $flatten
   * @param string $pidKey
   * @param string $idKey
   *
   * @return mixed
   */
  public static function buildTree($flatten, $pidKey = 'pid', $idKey = 'id') {
    $parents = [];

    foreach ($flatten as $item){
      $parents[$item[$pidKey]][] = $item;
    }

    $fnBuilder = function($items, $parents, $idKey) use (&$fnBuilder) {

      foreach ($items as $position => $item) {

        $id = $item[$idKey];

        if(isset($parents[$id])) { //is the parent set
          $item['children'] = $fnBuilder($parents[$id], $parents, $idKey); //add children
        }

        //reset the value as children have changed
        $items[$position] = $item;
      }

      //return the item
      return $items;
    };

    $parent = !empty($parents[0]) ? $parents[0] : [];

    return $fnBuilder($parent, $parents, $idKey);
  }

}
